# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'mslib/msui/ui/ui_performance_settings.ui'
#
# Created by: PyQt5 UI code generator 5.9.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_PerformanceSettingsDialog(object):
    def setupUi(self, PerformanceSettingsDialog):
        PerformanceSettingsDialog.setObjectName("PerformanceSettingsDialog")
        PerformanceSettingsDialog.resize(319, 216)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.MinimumExpanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(PerformanceSettingsDialog.sizePolicy().hasHeightForWidth())
        PerformanceSettingsDialog.setSizePolicy(sizePolicy)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(PerformanceSettingsDialog)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.cbShowPerformance = QtWidgets.QCheckBox(PerformanceSettingsDialog)
        self.cbShowPerformance.setObjectName("cbShowPerformance")
        self.verticalLayout_2.addWidget(self.cbShowPerformance)
        spacerItem = QtWidgets.QSpacerItem(0, 0, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.MinimumExpanding)
        self.verticalLayout_2.addItem(spacerItem)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.label_5 = QtWidgets.QLabel(PerformanceSettingsDialog)
        font = QtGui.QFont()
        font.setBold(False)
        font.setWeight(50)
        self.label_5.setFont(font)
        self.label_5.setObjectName("label_5")
        self.horizontalLayout.addWidget(self.label_5)
        self.lbAircraftName = QtWidgets.QLabel(PerformanceSettingsDialog)
        font = QtGui.QFont()
        font.setBold(False)
        font.setWeight(50)
        self.lbAircraftName.setFont(font)
        self.lbAircraftName.setObjectName("lbAircraftName")
        self.horizontalLayout.addWidget(self.lbAircraftName)
        self.pbLoadPerformance = QtWidgets.QPushButton(PerformanceSettingsDialog)
        self.pbLoadPerformance.setObjectName("pbLoadPerformance")
        self.horizontalLayout.addWidget(self.pbLoadPerformance)
        self.verticalLayout_2.addLayout(self.horizontalLayout)
        spacerItem1 = QtWidgets.QSpacerItem(0, 0, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.MinimumExpanding)
        self.verticalLayout_2.addItem(spacerItem1)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.label = QtWidgets.QLabel(PerformanceSettingsDialog)
        font = QtGui.QFont()
        font.setBold(False)
        font.setWeight(50)
        self.label.setFont(font)
        self.label.setObjectName("label")
        self.horizontalLayout_2.addWidget(self.label)
        self.dteTakeoffTime = QtWidgets.QDateTimeEdit(PerformanceSettingsDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.dteTakeoffTime.sizePolicy().hasHeightForWidth())
        self.dteTakeoffTime.setSizePolicy(sizePolicy)
        self.dteTakeoffTime.setObjectName("dteTakeoffTime")
        self.horizontalLayout_2.addWidget(self.dteTakeoffTime)
        self.verticalLayout_2.addLayout(self.horizontalLayout_2)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.label_2 = QtWidgets.QLabel(PerformanceSettingsDialog)
        font = QtGui.QFont()
        font.setBold(False)
        font.setWeight(50)
        self.label_2.setFont(font)
        self.label_2.setObjectName("label_2")
        self.horizontalLayout_4.addWidget(self.label_2)
        self.dsbTakeoffWeight = QtWidgets.QDoubleSpinBox(PerformanceSettingsDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.dsbTakeoffWeight.sizePolicy().hasHeightForWidth())
        self.dsbTakeoffWeight.setSizePolicy(sizePolicy)
        self.dsbTakeoffWeight.setDecimals(0)
        self.dsbTakeoffWeight.setMaximum(9999999.0)
        self.dsbTakeoffWeight.setSingleStep(1000.0)
        self.dsbTakeoffWeight.setObjectName("dsbTakeoffWeight")
        self.horizontalLayout_4.addWidget(self.dsbTakeoffWeight)
        self.verticalLayout_2.addLayout(self.horizontalLayout_4)
        self.horizontalLayout_7 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_7.setObjectName("horizontalLayout_7")
        self.label_3 = QtWidgets.QLabel(PerformanceSettingsDialog)
        font = QtGui.QFont()
        font.setBold(False)
        font.setWeight(50)
        self.label_3.setFont(font)
        self.label_3.setObjectName("label_3")
        self.horizontalLayout_7.addWidget(self.label_3)
        self.dsbFuel = QtWidgets.QDoubleSpinBox(PerformanceSettingsDialog)
        self.dsbFuel.setDecimals(0)
        self.dsbFuel.setMaximum(9999999.0)
        self.dsbFuel.setSingleStep(1000.0)
        self.dsbFuel.setObjectName("dsbFuel")
        self.horizontalLayout_7.addWidget(self.dsbFuel)
        self.verticalLayout_2.addLayout(self.horizontalLayout_7)
        spacerItem2 = QtWidgets.QSpacerItem(20, 20, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.MinimumExpanding)
        self.verticalLayout_2.addItem(spacerItem2)
        self.buttonBox = QtWidgets.QDialogButtonBox(PerformanceSettingsDialog)
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.verticalLayout_2.addWidget(self.buttonBox)
        self.buttonBox.raise_()
        self.label_5.raise_()
        self.cbShowPerformance.raise_()

        self.retranslateUi(PerformanceSettingsDialog)
        self.buttonBox.accepted.connect(PerformanceSettingsDialog.accept)
        self.buttonBox.rejected.connect(PerformanceSettingsDialog.reject)
        QtCore.QMetaObject.connectSlotsByName(PerformanceSettingsDialog)

    def retranslateUi(self, PerformanceSettingsDialog):
        _translate = QtCore.QCoreApplication.translate
        PerformanceSettingsDialog.setWindowTitle(_translate("PerformanceSettingsDialog", "PerformanceSettingsDialog"))
        self.cbShowPerformance.setText(_translate("PerformanceSettingsDialog", "Show Performance"))
        self.label_5.setText(_translate("PerformanceSettingsDialog", "Aircraft:"))
        self.lbAircraftName.setText(_translate("PerformanceSettingsDialog", "Dummy"))
        self.pbLoadPerformance.setText(_translate("PerformanceSettingsDialog", "Load"))
        self.label.setText(_translate("PerformanceSettingsDialog", "Take off time"))
        self.label_2.setText(_translate("PerformanceSettingsDialog", "Takeoff weight (lb)"))
        self.label_3.setText(_translate("PerformanceSettingsDialog", "Fuel (lb)"))

