  {% extends "theme.html" %} {% block body %}
  <div class="module module-gallery">
    <div class="container">
      <div class="p-3 mb-2 bg-warning text-white">
        <div class="module-content">
          <h1 class="header" style=
          "color:white;padding-top: 45px;margin-bottom:30px;">
          Welcome to the</h1>
          <h1 class="headline text-center" style=
          "color:white;padding-top: 45px;margin-bottom:30px;">
          Mission Support System Collaboration Platform</h1>
        </div>
      </div>
    </div>{% endblock %}
  </div>
